#include "tsdb.h"

#include <clickhouse/client.h>
#include <fmt/format.h>

#include <iostream>

void tsdb() {
#ifdef NDEBUG
    std::cout << "tsdb/0.1.0: Hello World Release!\n";
#else
    std::cout << "tsdb/0.1.0: Hello World Debug!\n";
#endif

// ARCHITECTURES
#ifdef _M_X64
    std::cout << "  tsdb/0.1.0: _M_X64 defined\n";
#endif

#ifdef _M_IX86
    std::cout << "  tsdb/0.1.0: _M_IX86 defined\n";
#endif

#ifdef _M_ARM64
    std::cout << "  tsdb/0.1.0: _M_ARM64 defined\n";
#endif

#if __i386__
    std::cout << "  tsdb/0.1.0: __i386__ defined\n";
#endif

#if __x86_64__
    std::cout << "  tsdb/0.1.0: __x86_64__ defined\n";
#endif

#if __aarch64__
    std::cout << "  tsdb/0.1.0: __aarch64__ defined\n";
#endif

// Libstdc++
#if defined _GLIBCXX_USE_CXX11_ABI
    std::cout << "  tsdb/0.1.0: _GLIBCXX_USE_CXX11_ABI " << _GLIBCXX_USE_CXX11_ABI << "\n";
#endif

// MSVC runtime
#if defined(_DEBUG)
#if defined(_MT) && defined(_DLL)
    std::cout << "  tsdb/0.1.0: MSVC runtime: MultiThreadedDebugDLL\n";
#elif defined(_MT)
    std::cout << "  tsdb/0.1.0: MSVC runtime: MultiThreadedDebug\n";
#endif
#else
#if defined(_MT) && defined(_DLL)
    std::cout << "  tsdb/0.1.0: MSVC runtime: MultiThreadedDLL\n";
#elif defined(_MT)
    std::cout << "  tsdb/0.1.0: MSVC runtime: MultiThreaded\n";
#endif
#endif

// COMPILER VERSIONS
#if _MSC_VER
    std::cout << "  tsdb/0.1.0: _MSC_VER" << _MSC_VER << "\n";
#endif

#if _MSVC_LANG
    std::cout << "  tsdb/0.1.0: _MSVC_LANG" << _MSVC_LANG << "\n";
#endif

#if __cplusplus
    std::cout << "  tsdb/0.1.0: __cplusplus" << __cplusplus << "\n";
#endif

#if __INTEL_COMPILER
    std::cout << "  tsdb/0.1.0: __INTEL_COMPILER" << __INTEL_COMPILER << "\n";
#endif

#if __GNUC__
    std::cout << "  tsdb/0.1.0: __GNUC__" << __GNUC__ << "\n";
#endif

#if __GNUC_MINOR__
    std::cout << "  tsdb/0.1.0: __GNUC_MINOR__" << __GNUC_MINOR__ << "\n";
#endif

#if __clang_major__
    std::cout << "  tsdb/0.1.0: __clang_major__" << __clang_major__ << "\n";
#endif

#if __clang_minor__
    std::cout << "  tsdb/0.1.0: __clang_minor__" << __clang_minor__ << "\n";
#endif

#if __apple_build_version__
    std::cout << "  tsdb/0.1.0: __apple_build_version__" << __apple_build_version__ << "\n";
#endif

    // SUBSYSTEMS

#if __MSYS__
    std::cout << "  tsdb/0.1.0: __MSYS__" << __MSYS__ << "\n";
#endif

#if __MINGW32__
    std::cout << "  tsdb/0.1.0: __MINGW32__" << __MINGW32__ << "\n";
#endif

#if __MINGW64__
    std::cout << "  tsdb/0.1.0: __MINGW64__" << __MINGW64__ << "\n";
#endif

#if __CYGWIN__
    std::cout << "  tsdb/0.1.0: __CYGWIN__" << __CYGWIN__ << "\n";
#endif
}

void tsdb_print_vector(const std::vector<std::string>& strings) {
    for (std::vector<std::string>::const_iterator it = strings.begin(); it != strings.end(); ++it) {
        std::cout << "tsdb/0.1.0 " << *it << std::endl;
    }
}

namespace tsdb_cpp {

struct tsdb_entry_impl {
    tsdb_entry_impl(std::string_view remote) : client_(clickhouse::ClientOptions().SetHost(remote.data())) {
        // std::cout << "tsdb/0.1.0: tsdb_entry_impl constructor\n";
    }

    void create_table_inner(std::string_view table_name, std::string_view engine, bool can_order, std::string_view extras = "") {
        table_name_ = table_name;
        auto query  = fmt::format(
            "CREATE TABLE IF NOT EXISTS {} ("
             "metric_name LowCardinality(String) COMMENT 'Metric name',"
             "point_name LowCardinality(String) COMMENT 'Point name',"
             "tags Map(LowCardinality(String), LowCardinality(String)) COMMENT 'Point tags',"
             "value Float64 COMMENT 'Point value',"
             "nanoseconds Int64 COMMENT 'Point time in nanoseconds' Codec(DoubleDelta, LZ4)"
             ") ENGINE = {}",
            table_name, engine);
        if (can_order) {
            query += " ORDER BY (metric_name, point_name, nanoseconds)";
        }
        if (!extras.empty()) {
            query += " ";
            query += extras;
        }
        client_.Execute(query);
    }

    void drop_table_inner(std::string_view table_name) {
        auto query = fmt::format("DROP TABLE IF EXISTS {}", table_name);
        client_.Execute(query);
    }

    void set_active_table(std::string_view table_name) { table_name_ = table_name; }

    void set_metric_name(std::string_view metric_name) { metric_name_ = metric_name; }

    void insert_point(const point& p) {
        clickhouse::Block block;
        auto col1 = std::make_shared<clickhouse::ColumnString>();
        col1->Append(metric_name_);
        block.AppendColumn("metric_name", col1);
        auto col2 = std::make_shared<clickhouse::ColumnString>();
        col2->Append(p.name_);
        block.AppendColumn("point_name", col2);
        auto col3 = std::make_shared<clickhouse::ColumnFloat64>();
        col3->Append(p.value_);
        block.AppendColumn("value", col3);
        auto col4 = std::make_shared<clickhouse::ColumnInt64>();
        col4->Append(p.nanoseconds_);
        block.AppendColumn("nanoseconds", col4);
        client_.Insert(table_name_, block);
    }

    void insert_points(const std::vector<point>& points) {
        clickhouse::Block block;
        auto col1 = std::make_shared<clickhouse::ColumnString>();
        auto col2 = std::make_shared<clickhouse::ColumnString>();
        auto col3 = std::make_shared<clickhouse::ColumnFloat64>();
        auto col4 = std::make_shared<clickhouse::ColumnInt64>();
        for (const auto& p : points) {
            col1->Append(metric_name_);
            col2->Append(p.name_);
            col3->Append(p.value_);
            col4->Append(p.nanoseconds_);
        }
        block.AppendColumn("metric_name", col1);
        block.AppendColumn("point_name", col2);
        block.AppendColumn("value", col3);
        block.AppendColumn("nanoseconds", col4);
        client_.Insert(table_name_, block);
    }

private:
    clickhouse::Client client_;
    std::string table_name_;
    std::string metric_name_ = "";
};

tsdb_entry::tsdb_entry(std::string_view remote) : impl_(std::make_unique<tsdb_entry_impl>(remote)) {
}
tsdb_entry::~tsdb_entry() {
}
tsdb_entry::tsdb_entry(tsdb_entry&& other) : impl_(std::move(other.impl_)) {
}
void tsdb_entry::close() {
    impl_.reset();
}
void tsdb_entry::create_table() {
    impl_->create_table_inner("tsdb_cpp", "MergeTree()", true);
}
void tsdb_entry::create_memory_table() {
    impl_->create_table_inner("tsdb_cpp_memory", "Memory()", false);
}
void tsdb_entry::drop_table() {
    impl_->drop_table_inner("tsdb_cpp");
}
void tsdb_entry::drop_memory_table() {
    impl_->drop_table_inner("tsdb_cpp_memory");
}
void tsdb_entry::set_active_table(std::string_view table_name) {
    impl_->set_active_table(table_name);
}
void tsdb_entry::set_metric_name(std::string_view metric_name) {
    impl_->set_metric_name(metric_name);
}
void tsdb_entry::insert_point(const point& p) {
    impl_->insert_point(p);
}
void tsdb_entry::insert_points(const std::vector<point>& points) {
    impl_->insert_points(points);
}

}  // namespace tsdb_cpp
